# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from .errors.bad_request_error import BadRequestError
from .errors.conflict_error import ConflictError
from .errors.failed_dependency_error import FailedDependencyError
from .errors.internal_server_error import InternalServerError
from .errors.unauthorized_error import UnauthorizedError
from .types.construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
from .types.extract_codes_result import ExtractCodesResult
from .types.extract_request_config import ExtractRequestConfig
from .types.extract_request_system import ExtractRequestSystem
from .types.upload_request_format import UploadRequestFormat

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConstrueClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        file: str,
        revision: typing.Optional[float] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConstrueUploadCodeSystemResponse]:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction.
        Upon upload, construe generates embeddings for all of the codes in the code system and stores them in the vector database so you can
        subsequently use the code system for construe/extract and lang2fhir/create (coming soon!)

        Parameters
        ----------
        name : str
            Name of the code system

        version : str
            Version of the code system

        format : UploadRequestFormat
            Format of the uploaded file

        file : str
            The file contents as a base64-encoded string

        revision : typing.Optional[float]
            Optional revision number

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConstrueUploadCodeSystemResponse]
            Successfully uploaded code system
        """
        _response = self._client_wrapper.httpx_client.request(
            "construe/upload",
            method="POST",
            json={
                "name": name,
                "version": version,
                "revision": revision,
                "format": format,
                "file": file,
                "code_col": code_col,
                "desc_col": desc_col,
                "defn_col": defn_col,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConstrueUploadCodeSystemResponse,
                    parse_obj_as(
                        type_=ConstrueUploadCodeSystemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ExtractCodesResult]:
        """
        Converts natural language text into structured medical codes

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ExtractCodesResult]
            Successfully extracted codes
        """
        _response = self._client_wrapper.httpx_client.request(
            "construe/extract",
            method="POST",
            json={
                "text": text,
                "system": convert_and_respect_annotation_metadata(
                    object_=system, annotation=ExtractRequestSystem, direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ExtractRequestConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractCodesResult,
                    parse_obj_as(
                        type_=ExtractCodesResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawConstrueClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def upload_code_system(
        self,
        *,
        name: str,
        version: str,
        format: UploadRequestFormat,
        file: str,
        revision: typing.Optional[float] = OMIT,
        code_col: typing.Optional[str] = OMIT,
        desc_col: typing.Optional[str] = OMIT,
        defn_col: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConstrueUploadCodeSystemResponse]:
        """
        Upload a custom medical code system with codes and descriptions for use in code extraction.
        Upon upload, construe generates embeddings for all of the codes in the code system and stores them in the vector database so you can
        subsequently use the code system for construe/extract and lang2fhir/create (coming soon!)

        Parameters
        ----------
        name : str
            Name of the code system

        version : str
            Version of the code system

        format : UploadRequestFormat
            Format of the uploaded file

        file : str
            The file contents as a base64-encoded string

        revision : typing.Optional[float]
            Optional revision number

        code_col : typing.Optional[str]
            Column name containing codes (required for CSV format)

        desc_col : typing.Optional[str]
            Column name containing descriptions (required for CSV format)

        defn_col : typing.Optional[str]
            Optional column name containing long definitions (for CSV format)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConstrueUploadCodeSystemResponse]
            Successfully uploaded code system
        """
        _response = await self._client_wrapper.httpx_client.request(
            "construe/upload",
            method="POST",
            json={
                "name": name,
                "version": version,
                "revision": revision,
                "format": format,
                "file": file,
                "code_col": code_col,
                "desc_col": desc_col,
                "defn_col": defn_col,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConstrueUploadCodeSystemResponse,
                    parse_obj_as(
                        type_=ConstrueUploadCodeSystemResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def extract_codes(
        self,
        *,
        text: str,
        system: typing.Optional[ExtractRequestSystem] = OMIT,
        config: typing.Optional[ExtractRequestConfig] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ExtractCodesResult]:
        """
        Converts natural language text into structured medical codes

        Parameters
        ----------
        text : str
            Natural language text to extract codes from

        system : typing.Optional[ExtractRequestSystem]

        config : typing.Optional[ExtractRequestConfig]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ExtractCodesResult]
            Successfully extracted codes
        """
        _response = await self._client_wrapper.httpx_client.request(
            "construe/extract",
            method="POST",
            json={
                "text": text,
                "system": convert_and_respect_annotation_metadata(
                    object_=system, annotation=ExtractRequestSystem, direction="write"
                ),
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=ExtractRequestConfig, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ExtractCodesResult,
                    parse_obj_as(
                        type_=ExtractCodesResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 424:
                raise FailedDependencyError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    AgentChatResponse,
    AgentCreateRequest,
    AgentCreateRequestProvider,
    AgentDeleteResponse,
    AgentGetChatMessagesRequestOrder,
    AgentGetChatMessagesRequestRole,
    AgentGetChatMessagesResponse,
    AgentListResponse,
    AgentPromptsResponse,
    AgentResponse,
    AgentTemplate,
    AgentTemplateProvider,
    ChatMessageTemplate,
    ChatMessageTemplateRole,
    ChatSessionTemplate,
    JsonPatch,
    JsonPatchOperation,
    JsonPatchOperationOp,
    PromptTemplate,
    SuccessResponse,
)
from .errors import BadRequestError, ForbiddenError, InternalServerError, NotFoundError, UnauthorizedError
from . import prompts
from .prompts import PromptsDeleteResponse, PromptsListResponse

__all__ = [
    "AgentChatResponse",
    "AgentCreateRequest",
    "AgentCreateRequestProvider",
    "AgentDeleteResponse",
    "AgentGetChatMessagesRequestOrder",
    "AgentGetChatMessagesRequestRole",
    "AgentGetChatMessagesResponse",
    "AgentListResponse",
    "AgentPromptsResponse",
    "AgentResponse",
    "AgentTemplate",
    "AgentTemplateProvider",
    "BadRequestError",
    "ChatMessageTemplate",
    "ChatMessageTemplateRole",
    "ChatSessionTemplate",
    "ForbiddenError",
    "InternalServerError",
    "JsonPatch",
    "JsonPatchOperation",
    "JsonPatchOperationOp",
    "NotFoundError",
    "PromptTemplate",
    "PromptsDeleteResponse",
    "PromptsListResponse",
    "SuccessResponse",
    "UnauthorizedError",
    "prompts",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata


class FhirResourceMeta(UniversalBaseModel):
    """
    Metadata about the resource
    """

    version_id: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="versionId")] = None
    last_updated: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="lastUpdated")] = None
    profile: typing.Optional[typing.List[str]] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .auth.client import AsyncAuthClient, AuthClient
from .raw_client import AsyncRawAuthtokenClient, RawAuthtokenClient


class AuthtokenClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawAuthtokenClient(client_wrapper=client_wrapper)
        self.auth = AuthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawAuthtokenClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawAuthtokenClient
        """
        return self._raw_client


class AsyncAuthtokenClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawAuthtokenClient(client_wrapper=client_wrapper)
        self.auth = AsyncAuthClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawAuthtokenClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawAuthtokenClient
        """
        return self._raw_client



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import PromptsDeleteResponse, PromptsListResponse

__all__ = ["PromptsDeleteResponse", "PromptsListResponse"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from .search_concept import SearchConcept


class CohortResponse(UniversalBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the cohort analysis was successful
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message with details about the analysis
    """

    patient_ids: typing_extensions.Annotated[typing.Optional[typing.List[str]], FieldMetadata(alias="patientIds")] = (
        pydantic.Field(default=None)
    )
    """
    Array of patient IDs that match the cohort criteria
    """

    patient_count: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="patientCount")] = (
        pydantic.Field(default=None)
    )
    """
    Total number of patients in the cohort
    """

    queries: typing.Optional[typing.List[SearchConcept]] = pydantic.Field(default=None)
    """
    Individual search concepts that were identified and executed
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.request_options import RequestOptions
from ...types.mcp_server_tool_call_response import McpServerToolCallResponse
from ...types.mcp_server_tool_response import McpServerToolResponse
from .raw_client import AsyncRawToolsClient, RawToolsClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawToolsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawToolsClient
        """
        return self._raw_client

    def list(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerToolResponse:
        """
        Lists all MCP server tools for a specific MCP server

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP server to list tools for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolResponse
            Successfully listed MCP server tools

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.mcp_server.tools.list(
            mcp_server_id="mcp_server_id",
        )
        """
        _response = self._raw_client.list(mcp_server_id, request_options=request_options)
        return _response.data

    def get(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerToolResponse:
        """
        Gets a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolResponse
            Successfully retrieved MCP server tool

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.mcp_server.tools.get(
            mcp_server_tool_id="mcp_server_tool_id",
        )
        """
        _response = self._raw_client.get(mcp_server_tool_id, request_options=request_options)
        return _response.data

    def delete(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerToolResponse:
        """
        Deletes a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolResponse
            Successfully deleted MCP server tool

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.mcp_server.tools.delete(
            mcp_server_tool_id="mcp_server_tool_id",
        )
        """
        _response = self._raw_client.delete(mcp_server_tool_id, request_options=request_options)
        return _response.data

    def call(
        self,
        mcp_server_tool_id: str,
        *,
        arguments: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> McpServerToolCallResponse:
        """
        Calls a MCP server tool

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to call

        arguments : typing.Dict[str, typing.Optional[typing.Any]]
            Arguments to pass to the MCP server tool

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolCallResponse
            Successfully called MCP server tool

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.tools.mcp_server.tools.call(
            mcp_server_tool_id="mcp_server_tool_id",
            arguments={"title": "PhenoML Agent API"},
        )
        """
        _response = self._raw_client.call(mcp_server_tool_id, arguments=arguments, request_options=request_options)
        return _response.data


class AsyncToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawToolsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawToolsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawToolsClient
        """
        return self._raw_client

    async def list(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerToolResponse:
        """
        Lists all MCP server tools for a specific MCP server

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP server to list tools for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolResponse
            Successfully listed MCP server tools

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.mcp_server.tools.list(
                mcp_server_id="mcp_server_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list(mcp_server_id, request_options=request_options)
        return _response.data

    async def get(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerToolResponse:
        """
        Gets a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolResponse
            Successfully retrieved MCP server tool

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.mcp_server.tools.get(
                mcp_server_tool_id="mcp_server_tool_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(mcp_server_tool_id, request_options=request_options)
        return _response.data

    async def delete(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> McpServerToolResponse:
        """
        Deletes a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolResponse
            Successfully deleted MCP server tool

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.mcp_server.tools.delete(
                mcp_server_tool_id="mcp_server_tool_id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete(mcp_server_tool_id, request_options=request_options)
        return _response.data

    async def call(
        self,
        mcp_server_tool_id: str,
        *,
        arguments: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> McpServerToolCallResponse:
        """
        Calls a MCP server tool

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to call

        arguments : typing.Dict[str, typing.Optional[typing.Any]]
            Arguments to pass to the MCP server tool

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        McpServerToolCallResponse
            Successfully called MCP server tool

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.tools.mcp_server.tools.call(
                mcp_server_tool_id="mcp_server_tool_id",
                arguments={"title": "PhenoML Agent API"},
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.call(
            mcp_server_tool_id, arguments=arguments, request_options=request_options
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .create_workflow_request_fhir_provider_id import CreateWorkflowRequestFhirProviderId
from .create_workflow_response import CreateWorkflowResponse
from .decision_node_definition import DecisionNodeDefinition
from .execute_workflow_response import ExecuteWorkflowResponse
from .execute_workflow_response_results import ExecuteWorkflowResponseResults
from .lang2fhir_create_definition import Lang2FhirCreateDefinition
from .lang2fhir_search_definition import Lang2FhirSearchDefinition
from .list_workflows_response import ListWorkflowsResponse
from .step_operation import StepOperation
from .sub_workflow_definition import SubWorkflowDefinition
from .update_workflow_request_fhir_provider_id import UpdateWorkflowRequestFhirProviderId
from .workflow_config import WorkflowConfig
from .workflow_definition import WorkflowDefinition
from .workflow_graph import WorkflowGraph
from .workflow_response import WorkflowResponse
from .workflow_response_graph import WorkflowResponseGraph
from .workflow_step import WorkflowStep
from .workflow_step_summary import WorkflowStepSummary
from .workflow_step_summary_type import WorkflowStepSummaryType
from .workflow_step_type import WorkflowStepType
from .workflows_delete_response import WorkflowsDeleteResponse
from .workflows_get_response import WorkflowsGetResponse
from .workflows_update_response import WorkflowsUpdateResponse

__all__ = [
    "CreateWorkflowRequestFhirProviderId",
    "CreateWorkflowResponse",
    "DecisionNodeDefinition",
    "ExecuteWorkflowResponse",
    "ExecuteWorkflowResponseResults",
    "Lang2FhirCreateDefinition",
    "Lang2FhirSearchDefinition",
    "ListWorkflowsResponse",
    "StepOperation",
    "SubWorkflowDefinition",
    "UpdateWorkflowRequestFhirProviderId",
    "WorkflowConfig",
    "WorkflowDefinition",
    "WorkflowGraph",
    "WorkflowResponse",
    "WorkflowResponseGraph",
    "WorkflowStep",
    "WorkflowStepSummary",
    "WorkflowStepSummaryType",
    "WorkflowStepType",
    "WorkflowsDeleteResponse",
    "WorkflowsGetResponse",
    "WorkflowsUpdateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ....core.api_error import ApiError
from ....core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ....core.http_response import AsyncHttpResponse, HttpResponse
from ....core.jsonable_encoder import jsonable_encoder
from ....core.pydantic_utilities import parse_obj_as
from ....core.request_options import RequestOptions
from ...errors.bad_request_error import BadRequestError
from ...errors.forbidden_error import ForbiddenError
from ...errors.internal_server_error import InternalServerError
from ...errors.unauthorized_error import UnauthorizedError
from ...types.mcp_server_tool_call_response import McpServerToolCallResponse
from ...types.mcp_server_tool_response import McpServerToolResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawToolsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[McpServerToolResponse]:
        """
        Lists all MCP server tools for a specific MCP server

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP server to list tools for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[McpServerToolResponse]
            Successfully listed MCP server tools
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/{jsonable_encoder(mcp_server_id)}/list",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolResponse,
                    parse_obj_as(
                        type_=McpServerToolResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[McpServerToolResponse]:
        """
        Gets a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[McpServerToolResponse]
            Successfully retrieved MCP server tool
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/tool/{jsonable_encoder(mcp_server_tool_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolResponse,
                    parse_obj_as(
                        type_=McpServerToolResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def delete(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[McpServerToolResponse]:
        """
        Deletes a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[McpServerToolResponse]
            Successfully deleted MCP server tool
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/tool/{jsonable_encoder(mcp_server_tool_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolResponse,
                    parse_obj_as(
                        type_=McpServerToolResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def call(
        self,
        mcp_server_tool_id: str,
        *,
        arguments: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[McpServerToolCallResponse]:
        """
        Calls a MCP server tool

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to call

        arguments : typing.Dict[str, typing.Optional[typing.Any]]
            Arguments to pass to the MCP server tool

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[McpServerToolCallResponse]
            Successfully called MCP server tool
        """
        _response = self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/tool/{jsonable_encoder(mcp_server_tool_id)}/call",
            method="POST",
            json={
                "arguments": arguments,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolCallResponse,
                    parse_obj_as(
                        type_=McpServerToolCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawToolsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self, mcp_server_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[McpServerToolResponse]:
        """
        Lists all MCP server tools for a specific MCP server

        Parameters
        ----------
        mcp_server_id : str
            ID of the MCP server to list tools for

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[McpServerToolResponse]
            Successfully listed MCP server tools
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/{jsonable_encoder(mcp_server_id)}/list",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolResponse,
                    parse_obj_as(
                        type_=McpServerToolResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[McpServerToolResponse]:
        """
        Gets a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to retrieve

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[McpServerToolResponse]
            Successfully retrieved MCP server tool
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/tool/{jsonable_encoder(mcp_server_tool_id)}",
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolResponse,
                    parse_obj_as(
                        type_=McpServerToolResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def delete(
        self, mcp_server_tool_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[McpServerToolResponse]:
        """
        Deletes a MCP server tool by ID

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to delete

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[McpServerToolResponse]
            Successfully deleted MCP server tool
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/tool/{jsonable_encoder(mcp_server_tool_id)}",
            method="DELETE",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolResponse,
                    parse_obj_as(
                        type_=McpServerToolResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def call(
        self,
        mcp_server_tool_id: str,
        *,
        arguments: typing.Dict[str, typing.Optional[typing.Any]],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[McpServerToolCallResponse]:
        """
        Calls a MCP server tool

        Parameters
        ----------
        mcp_server_tool_id : str
            ID of the MCP server tool to call

        arguments : typing.Dict[str, typing.Optional[typing.Any]]
            Arguments to pass to the MCP server tool

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[McpServerToolCallResponse]
            Successfully called MCP server tool
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"tools/mcp-server/tool/{jsonable_encoder(mcp_server_tool_id)}/call",
            method="POST",
            json={
                "arguments": arguments,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    McpServerToolCallResponse,
                    parse_obj_as(
                        type_=McpServerToolCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .construe_upload_code_system_response import ConstrueUploadCodeSystemResponse
from .extract_codes_result import ExtractCodesResult
from .extract_request_config import ExtractRequestConfig
from .extract_request_config_chunking_method import ExtractRequestConfigChunkingMethod
from .extract_request_config_validation_method import ExtractRequestConfigValidationMethod
from .extract_request_system import ExtractRequestSystem
from .extracted_code_result import ExtractedCodeResult
from .upload_request_format import UploadRequestFormat

__all__ = [
    "ConstrueUploadCodeSystemResponse",
    "ExtractCodesResult",
    "ExtractRequestConfig",
    "ExtractRequestConfigChunkingMethod",
    "ExtractRequestConfigValidationMethod",
    "ExtractRequestSystem",
    "ExtractedCodeResult",
    "UploadRequestFormat",
]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .create_summary_request_fhir_resources import CreateSummaryRequestFhirResources
from .create_summary_request_mode import CreateSummaryRequestMode
from .create_summary_response import CreateSummaryResponse
from .create_summary_template_response import CreateSummaryTemplateResponse
from .error_response import ErrorResponse
from .fhir_bundle import FhirBundle
from .fhir_bundle_entry_item import FhirBundleEntryItem
from .fhir_resource import FhirResource
from .summary_delete_template_response import SummaryDeleteTemplateResponse
from .summary_get_template_response import SummaryGetTemplateResponse
from .summary_list_templates_response import SummaryListTemplatesResponse
from .summary_template import SummaryTemplate
from .summary_update_template_response import SummaryUpdateTemplateResponse

__all__ = [
    "CreateSummaryRequestFhirResources",
    "CreateSummaryRequestMode",
    "CreateSummaryResponse",
    "CreateSummaryTemplateResponse",
    "ErrorResponse",
    "FhirBundle",
    "FhirBundleEntryItem",
    "FhirResource",
    "SummaryDeleteTemplateResponse",
    "SummaryGetTemplateResponse",
    "SummaryListTemplatesResponse",
    "SummaryTemplate",
    "SummaryUpdateTemplateResponse",
]



# This file was auto-generated by Fern from our API Definition.

import typing

from .fhir_bundle import FhirBundle
from .fhir_resource import FhirResource

CreateSummaryRequestFhirResources = typing.Union[FhirResource, FhirBundle]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import (
    ConstrueUploadCodeSystemResponse,
    ExtractCodesResult,
    ExtractRequestConfig,
    ExtractRequestConfigChunkingMethod,
    ExtractRequestConfigValidationMethod,
    ExtractRequestSystem,
    ExtractedCodeResult,
    UploadRequestFormat,
)
from .errors import BadRequestError, ConflictError, FailedDependencyError, InternalServerError, UnauthorizedError

__all__ = [
    "BadRequestError",
    "ConflictError",
    "ConstrueUploadCodeSystemResponse",
    "ExtractCodesResult",
    "ExtractRequestConfig",
    "ExtractRequestConfigChunkingMethod",
    "ExtractRequestConfigValidationMethod",
    "ExtractRequestSystem",
    "ExtractedCodeResult",
    "FailedDependencyError",
    "InternalServerError",
    "UnauthorizedError",
    "UploadRequestFormat",
]



# This file was auto-generated by Fern from our API Definition.

import typing

import httpx
from .agent.client import AgentClient, AsyncAgentClient
from .authtoken.client import AsyncAuthtokenClient, AuthtokenClient
from .cohort.client import AsyncCohortClient, CohortClient
from .construe.client import AsyncConstrueClient, ConstrueClient
from .core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from .environment import phenomlEnvironment
from .fhir.client import AsyncFhirClient, FhirClient
from .fhir_provider.client import AsyncFhirProviderClient, FhirProviderClient
from .lang2fhir.client import AsyncLang2FhirClient, Lang2FhirClient
from .summary.client import AsyncSummaryClient, SummaryClient
from .tools.client import AsyncToolsClient, ToolsClient
from .workflows.client import AsyncWorkflowsClient, WorkflowsClient


class phenoml:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : phenomlEnvironment
        The environment to use for requests from the client. from .environment import phenomlEnvironment



        Defaults to phenomlEnvironment.DEFAULT



    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.Client]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phenoml import phenoml

    client = phenoml(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: phenomlEnvironment = phenomlEnvironment.DEFAULT,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.Client] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = SyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.Client(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.Client(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.agent = AgentClient(client_wrapper=self._client_wrapper)
        self.authtoken = AuthtokenClient(client_wrapper=self._client_wrapper)
        self.cohort = CohortClient(client_wrapper=self._client_wrapper)
        self.construe = ConstrueClient(client_wrapper=self._client_wrapper)
        self.fhir = FhirClient(client_wrapper=self._client_wrapper)
        self.fhir_provider = FhirProviderClient(client_wrapper=self._client_wrapper)
        self.lang2fhir = Lang2FhirClient(client_wrapper=self._client_wrapper)
        self.summary = SummaryClient(client_wrapper=self._client_wrapper)
        self.tools = ToolsClient(client_wrapper=self._client_wrapper)
        self.workflows = WorkflowsClient(client_wrapper=self._client_wrapper)


class Asyncphenoml:
    """
    Use this class to access the different functions within the SDK. You can instantiate any number of clients with different configuration that will propagate to these functions.

    Parameters
    ----------
    base_url : typing.Optional[str]
        The base url to use for requests from the client.

    environment : phenomlEnvironment
        The environment to use for requests from the client. from .environment import phenomlEnvironment



        Defaults to phenomlEnvironment.DEFAULT



    token : typing.Union[str, typing.Callable[[], str]]
    headers : typing.Optional[typing.Dict[str, str]]
        Additional headers to send with every request.

    timeout : typing.Optional[float]
        The timeout to be used, in seconds, for requests. By default the timeout is 60 seconds, unless a custom httpx client is used, in which case this default is not enforced.

    follow_redirects : typing.Optional[bool]
        Whether the default httpx client follows redirects or not, this is irrelevant if a custom httpx client is passed in.

    httpx_client : typing.Optional[httpx.AsyncClient]
        The httpx client to use for making requests, a preconfigured client is used by default, however this is useful should you want to pass in any custom httpx configuration.

    Examples
    --------
    from phenoml import Asyncphenoml

    client = Asyncphenoml(
        token="YOUR_TOKEN",
    )
    """

    def __init__(
        self,
        *,
        base_url: typing.Optional[str] = None,
        environment: phenomlEnvironment = phenomlEnvironment.DEFAULT,
        token: typing.Union[str, typing.Callable[[], str]],
        headers: typing.Optional[typing.Dict[str, str]] = None,
        timeout: typing.Optional[float] = None,
        follow_redirects: typing.Optional[bool] = True,
        httpx_client: typing.Optional[httpx.AsyncClient] = None,
    ):
        _defaulted_timeout = (
            timeout if timeout is not None else 60 if httpx_client is None else httpx_client.timeout.read
        )
        self._client_wrapper = AsyncClientWrapper(
            base_url=_get_base_url(base_url=base_url, environment=environment),
            token=token,
            headers=headers,
            httpx_client=httpx_client
            if httpx_client is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout, follow_redirects=follow_redirects)
            if follow_redirects is not None
            else httpx.AsyncClient(timeout=_defaulted_timeout),
            timeout=_defaulted_timeout,
        )
        self.agent = AsyncAgentClient(client_wrapper=self._client_wrapper)
        self.authtoken = AsyncAuthtokenClient(client_wrapper=self._client_wrapper)
        self.cohort = AsyncCohortClient(client_wrapper=self._client_wrapper)
        self.construe = AsyncConstrueClient(client_wrapper=self._client_wrapper)
        self.fhir = AsyncFhirClient(client_wrapper=self._client_wrapper)
        self.fhir_provider = AsyncFhirProviderClient(client_wrapper=self._client_wrapper)
        self.lang2fhir = AsyncLang2FhirClient(client_wrapper=self._client_wrapper)
        self.summary = AsyncSummaryClient(client_wrapper=self._client_wrapper)
        self.tools = AsyncToolsClient(client_wrapper=self._client_wrapper)
        self.workflows = AsyncWorkflowsClient(client_wrapper=self._client_wrapper)


def _get_base_url(*, base_url: typing.Optional[str] = None, environment: phenomlEnvironment) -> str:
    if base_url is not None:
        return base_url
    elif environment is not None:
        return environment.value
    else:
        raise Exception("Please pass in either base_url or environment to construct the client")



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

from .types import CohortResponse, SearchConcept
from .errors import BadRequestError, InternalServerError, UnauthorizedError

__all__ = ["BadRequestError", "CohortResponse", "InternalServerError", "SearchConcept", "UnauthorizedError"]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .mcp_server_response_data import McpServerResponseData


class McpServerResponse(UniversalBaseModel):
    success: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the MCP server was created successfully
    """

    message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Status message
    """

    data: typing.Optional[McpServerResponseData] = pydantic.Field(default=None)
    """
    MCP server data
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from .raw_client import AsyncRawCohortClient, RawCohortClient
from .types.cohort_response import CohortResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class CohortClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawCohortClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawCohortClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawCohortClient
        """
        return self._raw_client

    def analyze(self, *, text: str, request_options: typing.Optional[RequestOptions] = None) -> CohortResponse:
        """
        Converts natural language text into structured FHIR search queries for patient cohort analysis

        Parameters
        ----------
        text : str
            Natural language text describing patient cohort criteria

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CohortResponse
            Successfully analyzed cohort criteria

        Examples
        --------
        from phenoml import phenoml

        client = phenoml(
            token="YOUR_TOKEN",
        )
        client.cohort.analyze(
            text="female patients over 65 with diabetes but not hypertension",
        )
        """
        _response = self._raw_client.analyze(text=text, request_options=request_options)
        return _response.data


class AsyncCohortClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawCohortClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawCohortClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawCohortClient
        """
        return self._raw_client

    async def analyze(self, *, text: str, request_options: typing.Optional[RequestOptions] = None) -> CohortResponse:
        """
        Converts natural language text into structured FHIR search queries for patient cohort analysis

        Parameters
        ----------
        text : str
            Natural language text describing patient cohort criteria

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CohortResponse
            Successfully analyzed cohort criteria

        Examples
        --------
        import asyncio

        from phenoml import Asyncphenoml

        client = Asyncphenoml(
            token="YOUR_TOKEN",
        )


        async def main() -> None:
            await client.cohort.analyze(
                text="female patients over 65 with diabetes but not hypertension",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.analyze(text=text, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class ExtractRequestSystem(UniversalBaseModel):
    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Code system name. Can be a built-in system or a custom system name.
    
    Built-in systems:
    * SNOMED_CT_US_LITE - version 20240901
    * RXNORM - version 11042024
    * ICD-10-CM - version 2025
    * ICD-10-PCS - version 2025
    * LOINC - version 2.78
    * HPO - version 2025
    * CPT - version 2025
    
    Custom systems:
    * Any valid system name configured in your environment. Must have self-hosted construe module.
    """

    version: typing.Optional[str] = pydantic.Field(default=None)
    """
    Code system version. Must match the version available in your environment.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class FhirProviderSandboxInfo(UniversalBaseModel):
    """
    Information returned for sandbox FHIR providers.
    """

    id: typing.Optional[str] = pydantic.Field(default=None)
    """
    Unique identifier for the FHIR provider
    """

    name: typing.Optional[str] = pydantic.Field(default=None)
    """
    Display name for the FHIR provider
    """

    description: typing.Optional[str] = pydantic.Field(default=None)
    """
    Optional description of the FHIR provider
    """

    provider: typing.Optional[typing.Literal["sandbox"]] = pydantic.Field(default=None)
    """
    Provider type (always "sandbox" for this schema)
    """

    is_active: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the FHIR provider is active
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
